'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var nodeFetch = require('node-fetch');
var mimeTypes = require('mime-types');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var fs__default = /*#__PURE__*/_interopDefaultLegacy(fs);

/**
 * ignore
 *
 * @param {any} reject ignore
 * @param {any} fct ignore
 * @returns {any} ignore
 */
function __wrapReject (reject, fct) {
  try {
    return fct()
  } catch (e) {
    reject(e);
  }
}

/**
 * Do stuff
 *
 * @param {string|URL|Request} resource resource
 * @param {object} init init
 * @returns {Promise<Response>} something
 */
async function fetchFile (resource, init) {
  const request = new nodeFetch.Request(resource, init);
  return new Promise((resolve, reject) => {
    const resolveError = (error) => {
      const errorMessage = error.message;
      const buffer = Buffer.from(errorMessage);
      const status = (() => {
        if (error.code === 'ENOENT') {
          return 404
        } if (error.code === 'EACCES') {
          return 403
        } else {
          return 500
        }
      })();
      const statusText = (() => {
        if (status === 404) {
          return 'Not Found'
        } else if (status === 403) {
          return 'Forbidden'
        } else { // 500
          return 'Internal Server Error'
        }
      })();
      resolve(new nodeFetch.Response(buffer, {
        url: request.url,
        status,
        statusText,
        size: buffer.length,
        headers: new nodeFetch.Headers({
          'Content-Length': `${buffer.length}`,
          'Content-Type': 'text/plain'
        }),
        timeout: request.timeout
      }));
    };
    __wrapReject(resolveError, () => {
      const url = request.url;
      const match = url.match(/^file:\/\/(.*)$/);
      const filePath = match ? match[1] : url;

      const readStream = fs__default['default'].createReadStream(filePath);
      readStream.on('open', () => {
        __wrapReject(resolveError, async () => {
          const stat = await fs__default['default'].promises.stat(filePath);
          const response = new nodeFetch.Response(readStream, {
            url: request.url,
            status: 200,
            statusText: 'OK',
            size: stat.size,
            headers: new nodeFetch.Headers({
              'Content-Length': stat.size,
              'Content-Type': mimeTypes.lookup(filePath) || 'application/octet-stream'
            }),
            timeout: request.timeout
          });
          resolve(response);
        });
      });
      readStream.on('error', resolveError);
    });
  })
}

exports.fetchFile = fetchFile;
